#!/usr/bin/env python3
import pathlib, pandas, math, sys,  re


def aggregate_results(result_files, output):
    result_df = []
    for r in result_files:
        try:
            tmp = pandas.read_csv(r, sep = '\t')
            result_df.append(tmp)
        except:
            print(f"{r} has no columns")
    if result_df == []:
        print("No valid result files found")
        sys.exit(1)
    result_df = pandas.concat(result_df)
    result_df = result_df.fillna('')

    result_df.to_csv(f"{output}.txt", sep = '\t', index = False)


result_files = []

for i in sys.argv[2:]:
    l = i.strip('[,]')
    # print(l)
    result_files.append(l)

aggregate_results(result_files = result_files, output = sys.argv[1])



from bohra.launcher.Utils import CustomFormatter, _extract_tool_list, _check_path
import logging
import subprocess
import pathlib
import os
import json

# Logger
LOGGER =logging.getLogger(__name__) 
LOGGER.setLevel(logging.DEBUG)
ch = logging.StreamHandler()
ch.setLevel(logging.DEBUG)
ch.setFormatter(CustomFormatter())
fh = logging.FileHandler('bohra.log')
fh.setLevel(logging.DEBUG)
formatter = logging.Formatter('[%(levelname)s:%(asctime)s] %(message)s', datefmt='%Y-%m-%d %I:%M:%S %p') 
fh.setFormatter(formatter)
LOGGER.addHandler(ch) 
LOGGER.addHandler(fh)


def _run_cmd(cmd:list, check:bool=False)-> bool:

    """
    Run a command and log output.
    """
    if not check:
        LOGGER.info(f"Running command: {' '.join(cmd)}")
    process = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, encoding='utf-8')
    while process.poll() is None:
        l = process.stdout.readline().strip() # This blocks until it receives a newline.
        if len(l.split()) > 0 and not check:
            LOGGER.info(f"{l}")
    if process.returncode != 0:
        print(process.args)
        LOGGER.warning(f"Error running command: {' '.join(cmd)}")
        if len( process.stdout.read().strip()) > 0:
            LOGGER.warning(f"{process.stdout.read()}")
        return False
    else:
        if not check:
            LOGGER.info(f"Command completed successfully")
    return True


    
def _check_envs(cfg:dict)->bool:
    """
    Check if conda environments directory exists.
    """
    bohra_env_dir = f"{os.getenv('CONDA_PREFIX')}"
    target_envs_dir = f"{bohra_env_dir}/bohra_conda_envs"
    for env in cfg:

        for dep in cfg[env]:
            env_name = f"{target_envs_dir}/{env}"
            if not pathlib.Path(env_name).exists():
                LOGGER.warning(f"Conda environment {env} not found at {env_name}.")
                return False
            cmd = ["conda", "run", "-p", env_name]
            cmd.extend(dep.split())
            if not _run_cmd(cmd, check=True):
                LOGGER.critical(f"Dependency {env} not installed properly.")
                return False
            else:
                LOGGER.info(f"{env} environment is found and {dep.split()[0]} appear to be installed properly.")
    return True

def check_conda_version(version:str)->bool:

    version_parts = version.split('.')
    if version_parts[0].isdigit() and int(version_parts[0]) >= 25:
        LOGGER.info("Conda version 25 or higher has been found.")
        return True
    else:
        LOGGER.critical(f"Conda version {version} has been found. Bohra is designed for conda version 25 or higher. Unexpected behaviour may result - ideally update conda.")
        return False


def get_conda_version()->str:
    """
    Get conda version.
    """
    cmd = ["conda", "--version"]
    process = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, encoding='utf-8')
    output, _ = process.communicate()
    if process.returncode != 0:
        LOGGER.warning(f"Error getting conda version.")
        return ""
    else:
        version = output.strip().split()[-1]
        LOGGER.info(f"Conda version: {version}")
        return version

def _install_envs(cfg:dict, envs_path:str, env:str="all",force_reinstall:bool=False)->bool:
    """
    Install conda environments.
    """
    # checking if mamba is installed
    installer = "conda"
    version_25 = check_conda_version(get_conda_version())
    proc = subprocess.run(["which", "mamba"], capture_output=True, text=True)
    if proc.returncode == 0:
        installer = "mamba"
        LOGGER.info("Using mamba to install dependencies.")
    else:
        LOGGER.info("Using conda to install dependencies.")
    # check if envs path exists
    bohra_env_dir = f"{os.getenv('CONDA_PREFIX')}"
    if pathlib.Path(bohra_env_dir).exists():
        target_envs_dir = f"{bohra_env_dir}/bohra_conda_envs"
        LOGGER.info(f"Dependency environments will be installed in: {target_envs_dir}")
    else:
        LOGGER.critical("CONDA_PREFIX environment variable not set. Please activate a conda environment before installing dependencies.")
        raise SystemExit
    if env != "all":
        cfg = {env: cfg[env]}
    for env_name in cfg:
        if _check_envs(cfg={env_name: cfg[env_name]}) and not force_reinstall:
            LOGGER.info(f"Environment {env_name} already installed and force_reinstall is False. Skipping installation.")
            # continue
        else:
            LOGGER.info(f"Setting up environment: {env_name}")
            yml_file = f"{envs_path}/{env_name}.yml"
            if not pathlib.Path(yml_file).exists():
                LOGGER.critical(f"Environment file {yml_file} does not exist.")
                raise SystemExit
            cmd = [installer, "env", "create", "-y", "-f", yml_file, "-p", f"{target_envs_dir}/{env_name}"] if version_25 else [installer, "env", "create", "-f", yml_file, "-p", f"{target_envs_dir}/{env_name}"]
            if force_reinstall and not version_25:
                cmd.append("--force")
            else:
                subprocess.run([installer, "env", "remove", "-p", f"{target_envs_dir}/{env_name}", "-all"], stdout=subprocess.PIPE, stderr=subprocess.PIPE, text=True)
            if not _run_cmd(cmd):
                return False
            else:
                if _check_envs( cfg={env_name: cfg[env_name]}):
                    LOGGER.info(f"Environment {env_name} installed and verified successfully.")
                else:
                    LOGGER.critical(f"Environment {env_name} failed verification after installation.")
                    return False
    LOGGER.info("All dependencies installed successfully and verified.")
    return True


def dependencies(_action:str = "install",
                       envs:str=f"{pathlib.Path(__file__).parent.parent}/environments",
                       config:str=f"{pathlib.Path(__file__).parent.parent}/config/dependencies.json",
                       kwargs: dict = {})->int:
    """
    Install bohra dependencies.
    """
    script_path = f"{pathlib.Path(__file__).parent}"
    dep_cfg = _extract_tool_list(config, tool=kwargs.get('tool', 'all'))
    actions = ['install', 'update', 'check']
    
    LOGGER.info(f"Will now try {_action} dependencies. Please be patient this may take some time!!... Maybe get coffee.")
    if _action not in actions:
        LOGGER.critical(f"Invalid action: {_action}. Must be one of {actions}.")
        raise SystemExit
    if _action == 'check':
        if not _check_envs(dep_cfg):
            tool = kwargs.get('tool', 'all')
            LOGGER.critical(f"There are missing or improperly installed dependencies. Please run 'bohra deps install' to install.")
            return 1
        else:
            LOGGER.info("All dependencies are installed properly.")
            return 0
    elif _action in ['install', 'update']:
        force_reinstall = True if _action == 'update' else False
        env_to_install = kwargs.get('tool', 'all')
        if not _install_envs(dep_cfg, envs, env=env_to_install, force_reinstall=force_reinstall):
            LOGGER.critical("Error installing dependencies.")
            return 1
        else:
            return 0
            # else:
            #     LOGGER.critical("Some dependencies failed verification after installation.")
            #     return 1

def _get_db_config(config:str=f"{pathlib.Path(__file__).parent.parent}/config/databases.json")->dict:
    """
    Get database configuration from json file.
    """
    if not pathlib.Path(config).exists():
        LOGGER.critical(f"Database configuration file {config} does not exist.")
        raise SystemExit
    with open(config, 'r') as f:
        db_cfg = json.load(f)
    return db_cfg

def _construct_options_string(options:dict)->str:
    """
    Construct options string from dictionary.
    """
    options_list = []
    for val in options:
    
        line = f"{val['key']} - {val['name']} (size {val['size']})"
        options_list.append(line)
    return "\n".join(options_list)

def _check_databases(db_install:bool=False,
                    database_path:str="",
                    config:str=f"{pathlib.Path(__file__).parent.parent}/config/databases.json")->int:
    """
    Check that required databases are installed.
    """
    script_path = f"{pathlib.Path(__file__).parent}"
    LOGGER.info(f"Will now check databases required for Bohra. Please be patient this may take some time!!... Maybe get coffee.")
    db_check_cmd = 'get' if db_install else 'check'
    db_cfg = _get_db_config(config=config)
    
    for essential in db_cfg["essential"]:
        var = os.getenv(essential)
        
        if var is None or not pathlib.Path(var).exists():
            LOGGER.warning(f"Essential database {essential} not found or environment variable not set.")
            if var == "KRAKEN2_DB_PATH":
                LOGGER.info(f"You will need to provide the path to the Kraken2 database for Bohra run speciation and other species-specific analyses.")
            if not db_install:
                LOGGER.warning(f"If you wish to install missing databases, please run 'bohra init_databases --setup_databases --database_path <path>'.")
            else:
                LOGGER.info(f"You will need to select one of the following options to setup the database:")
                options_str = _construct_options_string(db_cfg["essential"][essential]["urls"])
                db_choice = input(f"Select an option to setup {essential}:\n{options_str}\nEnter option key: ")
                selected_url = None
                for option in db_cfg["essential"][essential]["urls"]:
                    if str(option["key"]) == db_choice:
                        selected_url = option["url"]
                        target = option["target"]
                        break
                if selected_url is None and db_choice != "0":
                    LOGGER.critical(f"Invalid option selected: {db_choice}. Exiting.")
                    raise SystemExit
                elif db_choice == "0":
                    existing_path = input(f"Please provide the existing path to the {essential} database: ")
                    if _check_path(existing_path):
                        new_essential_path = existing_path
                        # LOGGER.info(f"Environment variable {essential} set to {existing_path}.")
                        continue
                    else:
                        LOGGER.critical(f"Provided path {existing_path} is invalid. Exiting.")
                        raise SystemExit
                if database_path == "":
                    database_path = input(f"Please provide the path to download and setup the {essential} database: ")
                if _check_path(database_path):
                    cmd = f"wget --continue -O {database_path}/{target} {selected_url}"
                    LOGGER.info(f"Running : {cmd}")
                    if not _run_cmd(cmd.split()):
                        LOGGER.critical(f"Error downloading database from {selected_url}. Exiting.")
                        raise SystemExit
                    else:
                        new_essential_path = f"{database_path}/{pathlib.Path(target).name.split('.')[0]}"
                        LOGGER.info(f"Uncompressing database {target}. This may take some time...")
                        cmd = f"tar -xvzf {database_path}/{target} -C {database_path}"
                        LOGGER.info(f"Running : {cmd}")
                        if not _run_cmd(cmd.split()):
                            LOGGER.critical(f"Error uncompressing database {target}. Exiting.")
                            raise SystemExit
                        LOGGER.info(f"Database {essential} downloaded.")
                LOGGER.info(f"Setting environment variable {essential} to {new_essential_path}.")
                cmd = f"conda env config vars set {essential}={new_essential_path} && conda deactivate && conda activate {os.getenv('CONDA_PREFIX')}"
                LOGGER.info(f"Running : {cmd}")
                # if not _run_cmd(cmd.split()):
                #     LOGGER.critical(f"Error setting environment variable {essential}. Exiting.")
                #     raise SystemExit
        else:
            LOGGER.info(f"Database {essential} found at {var}.")
    for other in db_cfg["non_essential"]:
        var = os.getenv(other)
        if var is None or not pathlib.Path(var).exists():
            LOGGER.warning(f"Optional database {other} not found or environment variable not set.")
            LOGGER.warning(f"Bohra will use the default databases that come with the tools for analyses requiring {other}. Otherwise you may be able to provide this at runtime.")
        else:
            LOGGER.info(f"Database {other} found at {var}.")
   


#!/usr/bin/env python3
import json
import sys
import pandas as pd
from argparse import ArgumentParser
"""
This script was cloned from https://github.com/katholt/sonneityping 10th May 2021
"""
def get_arguments():
    parser = ArgumentParser(description='Parse mykrobe predict JSON files')

    # job submission options
    parser.add_argument('--jsons', required=True, nargs='+', help='JSON files output from mykrobe predict')
    parser.add_argument('--alleles', required=True, help='alleles.txt file that defines each genotype, its marker, and the human-readable name')
    parser.add_argument('--prefix', required=True, help='prefix for output files')

    return parser.parse_args()

def extract_qrdr_info(genome_data, genome_name, spp_call):

    # set up empty dict for final output
    qrdr_out_dict = {}
    # make a list of all possible mutations
    qrdr_possible = ['parC_S80I', 'gyrA_S83L', 'gyrA_S83A', 'gyrA_D87G', 'gyrA_D87N', 'gyrA_D87Y']

    # can only extract qrdr info if sample is sonnei, otherwise set 'unknown' for all calls
    if spp_call == "Shigella_sonnei":
        try:
            qrdr_data = genome_data["susceptibility"]["quinolones"]
        # if nothiing has been called, just set everything to NA
        except KeyError:
            qrdr_out_dict['genome'] = [genome_name]
            for allele in qrdr_possible:
                qrdr_out_dict[allele] = ['NA']
            qrdr_out_dict['num QRDR'] = ['NA']
            # make table
            qrdr_out_df = pd.DataFrame(qrdr_out_dict, columns=['genome'] + qrdr_possible + ['num QRDR'])
            return qrdr_out_df
    else:
        qrdr_out_dict['genome'] = [genome_name]
        for allele in qrdr_possible:
            qrdr_out_dict[allele] = ['NA']
        qrdr_out_dict['num QRDR'] = ['NA']
        # make table
        qrdr_out_df = pd.DataFrame(qrdr_out_dict, columns=['genome'] + qrdr_possible + ['num QRDR'])
        return qrdr_out_df

    # set up the list of calls in this genome
    qrdr_calls = []
    # only need to parse if the predict is R - if it's S all values will be 0
    if qrdr_data["predict"] == "R":
        calls = qrdr_data["called_by"]
        # loop through each mutation
        for mutation in calls:
            # get the mutation name and add it to our list of calls
            qrdr_calls.append(mutation.split('-')[0])
    # now create our pandas dataframe for this genome
    qrdr_out_dict['genome'] = [genome_name]
    num_qrdr_calls = 0
    for allele in qrdr_possible:
        if allele in qrdr_calls:
            qrdr_out_dict[allele] = [1]
            num_qrdr_calls += 1
        else:
            qrdr_out_dict[allele] = [0]
    # add column with total number of qrdr calls
    qrdr_out_dict['num QRDR'] = [num_qrdr_calls]
    # make table
    qrdr_out_df = pd.DataFrame(qrdr_out_dict, columns=['genome'] + qrdr_possible + ['num QRDR'])
    return qrdr_out_df

def inspect_calls(full_lineage_data):
    # for all the genotypes that mykrobe is calling, inspect the values listed next to the actual call heirarhcy
    genotype_details = full_lineage_data['calls_summary']
    genotype_list = list(genotype_details.keys())
    # intialise empty values for best score and corresponding genotype
    best_score = 0
    best_genotype = None
    # node support is the number of 'good nodes' called by mykrobe
    # needs to be X/Y, where Y is the total number of levels at that genotype
    node_support = None
    # loop through each genotype
    for genotype in genotype_list:
        # the maximum score we can get is the total depth of the heirarchy
        max_score = genotype_details[genotype]['tree_depth']
        # the actual score is a sum of the values within each levels call
        actual_score = sum(list(genotype_details[genotype]['genotypes'].values()))
        # if actual score is equal to the max score, then this is the best genotype
        # BUT WE NEED TO DEAL WITH INSTANCES WHERE WE MIGHT HAVE A GREAT CALL 3.7.25 say AND A LESS GREAT CALL 3.6.1.1.2 say BUT BECACUSE THE HEIRARHCY IS BIGGER FOR 3.6.1.1.2 WE INADVERTANTLY CALL THAT
        if actual_score == max_score:
            best_score = actual_score
            best_genotype = genotype
            #node_support = genotype_details[best_genotype]['good_nodes']
        # if actual score is < max score, but is greater than the current best score,
        # then this is our best genotype for the moment
        elif actual_score < max_score and actual_score > best_score:
            best_score = actual_score
            best_genotype = genotype
            #node_support = genotype_details[best_genotype]['good_nodes']

    # determine any quality issues and split them amongst our various columns
    # if the node support for the best genotype is not '1' at all positions, we need to report that
    best_calls = genotype_details[best_genotype]['genotypes']
    # get a list of all the calls for calculating confidence later
    best_calls_vals = list(best_calls.values())
    poorly_supported_markers = []
    # dict that keeps percentage supports, key=percent support; value=level
    lowest_within_genotype_percents = {}
    # list that gives supports for all markers in best call
    final_markers = []
    for level in best_calls.keys():
        # regardless of the call, get info
        call_details = full_lineage_data['calls'][best_genotype][level]
        # check that there is something there
        if call_details:
            # need to do this weird thing to grab the info without knowing the key name
            call_details = call_details[list(call_details.keys())[0]]
            ref = call_details['info']['coverage']['reference']['median_depth']
            alt = call_details['info']['coverage']['alternate']['median_depth']
            # calculate percent support for marker
            try:
                percent_support = alt / (alt + ref)
            except ZeroDivisionError:
                percent_support = 0
            marker_string = level + ' (' + str(best_calls[level]) + '; ' + str(alt) + '/' + str(ref) + ')'
            final_markers.append(marker_string)
        # if the value is null, just report 0 (indicates that no SNV detected, either ref or alt?)
        else:
            # note we do not have a markers for lineage5.1 so don't report this as 0
            if level != 'lineage5.1':
                lowest_within_genotype_percents[0] = level
                marker_string = level + ' (0)'
                # its returned null so is therefore by definition poorly supported
                poorly_supported_markers.append(marker_string)
                # add to the final markers list too
                final_markers.append(marker_string)
        # if call is 1 then that is fine, count to determine confidence later
        # if call is 0.5, then get info
        # if call is 0, there will be no info in the calls section, so just report 0s everywhere
        if best_calls[level] < 1 and level != 'lineage5.1':
            # then it must be a 0 or a 0.5
            # report the value (0/0.5), and also the depth compared to the reference
            lowest_within_genotype_percents[percent_support] = level
            # add this to the list of poorly supported markers
            poorly_supported_markers.append(marker_string)

    # determining final confidence is based ONLY on the actual genotype, not incongruent genotype calls
    # if there is a lineage5.1 in the call, then we need to remove one of the 0s in best_calls_vals
    if 'lineage5.1' in best_calls.keys():
        best_calls_vals.remove(0)
    # strong = quality 1 for all calls
    if best_calls_vals.count(0) == 0 and best_calls_vals.count(0.5) == 0:
        confidence = 'strong'
        lowest_support_val = ''
    # moderate = quality 1 for all calls bar 1 (which must be quality 0.5 with percent support > 0.5)
    elif best_calls_vals.count(0) == 0 and best_calls_vals.count(0.5) == 1:
        if min(lowest_within_genotype_percents.keys()) > 0.5:
            confidence = 'moderate'
        else:
            confidence = 'weak'
        lowest_support_val = round(min(lowest_within_genotype_percents.keys()), 3)
    # weak = more than one quality < 1, or the single 0.5 call is < 0.5% support
    else:
        confidence = 'weak'
        lowest_support_val = round(min(lowest_within_genotype_percents.keys()), 3)

    # make a list of all possible quality issues (incongruent markers, or not confident calls within the best geno)
    non_matching_markers = []
    non_matching_supports = []
    # we now want to report any additional markers that aren't congruent with our best genotype
    #ie if 3.6.1 is the best genotype, but we also have a 3.7.29 call, we need to report the 3.7 and 3.29 markers as incongruent
    if len(genotype_list) > 1:
        # remove the best genotype from the list
        genotype_list.remove(best_genotype)
        # loop through each incongruent phenotype
        for genotype in genotype_list:
            # extract the calls for that genotype
            other_calls = genotype_details[genotype]['genotypes']
            # for every call in our BEST calls, we're only interested
            # in calls that are incongruent
            for call in other_calls.keys():
                if call not in best_calls.keys():
                    call_info = full_lineage_data['calls'][genotype][call]
                    # check that there is something there (if the value is null, don't report it for incongruent calls)
                    if call_info:
                        # need to do this weird thing to grab the info without knowing the key name
                        call_info = call_info[list(call_info.keys())[0]]
                        ref_depth = call_info['info']['coverage']['reference']['median_depth']
                        alt_depth = call_info['info']['coverage']['alternate']['median_depth']
                        # only keep the call if the alternate has a depth of > 1
                        # this is because mykrobe fills in intermediate levels of the heirarchy with 0s
                        # if a lower level SNV marker is detected
                        if alt_depth >= 1:
                            percent_support = alt_depth / (alt_depth + ref_depth)
                            non_matching_supports.append(percent_support)
                            marker_string = call + ' (' + str(other_calls[call]) + '; ' + str(alt_depth) + '/' + str(ref_depth) + ')'
                            non_matching_markers.append(marker_string)

    # get max value for non matching supports, if empty, return ''
    if len(non_matching_supports) > 0:
        max_non_matching = round(max(non_matching_supports), 3)
    else:
        max_non_matching = ''

    return best_genotype, confidence, lowest_support_val, poorly_supported_markers, max_non_matching, non_matching_markers, final_markers

def extract_lineage_info(lineage_data, genome_name, lineage_name_dict):

    # first, extract phylo spp information - if the spp is not sonnei then don't proceed
    spp_data = lineage_data['species']
    spp_call = list(spp_data.keys())[0]
    # if spp is unknown, then this is not sonnei, exit this function
    if spp_call == "Unknown":
        out_dict = {'genome':[genome_name], 'species':['not S. sonnei'], 'name':['NA'],'final genotype':['NA'],'confidence':['NA'],'lowest support for genotype marker':[''], 'poorly supported markers':[''], 'node support':[''], 'max support for additional markers':[''], 'additional markers':['']}
        out_df = pd.DataFrame(out_dict, columns=['genome', 'final genotype', 'name', 'confidence', 'lowest support for genotype marker', 'poorly supported markers', 'node support', 'max support for additional markers', 'additional markers'])
        return out_df, spp_call
    else:
        # if it is sonnei, then get the percentage
        spp_percentage = spp_data["Shigella_sonnei"]["percent_coverage"]
        # if the percentage is <90, then exit this function as it's likely not sonnei
        if spp_percentage < 90:
            out_dict = {'genome':[genome_name], 'species':['not S. sonnei'], 'name':['NA'],'final genotype':['NA'],'confidence':['NA'],'lowest support for genotype marker':[''], 'poorly supported markers':[''], 'node support':[''], 'max support for additional markers':[''], 'additional markers':['']}
            out_df = pd.DataFrame(out_dict, columns=['genome', 'final genotype', 'name', 'confidence', 'lowest supprot for genotype marker', 'poorly supported markers', 'node support', 'max support for additional markers', 'additional markers'])
            return out_df, "Unknown"

    # if we are sonnei, then continue
    # set up dictionary for final table output
    lineage_out_dict = {'genome': [genome_name]}

    # this try/except statement deals with instances where for some reason there is no lineage output
    # in the json file
    try:
        genotype_calls = lineage_data['lineage']['lineage']
    except KeyError:
        out_dict = {'genome':[genome_name], 'species':['S. sonnei'], 'name':['NA'],'final genotype':['uncalled'],'confidence':['NA'],'lowest support for genotype marker':[''], 'poorly supported markers':[''], 'max support for additional markers':[''], 'additional markers':[''], 'node support':['']}
        out_df = pd.DataFrame(out_dict, columns=['genome', 'final genotype', 'name', 'confidence', 'lowest support for genotype marker', 'poorly supported markers', 'node support', 'max support for additional markers', 'additional markers'])
        return out_df, spp_call
    # if there are no calls, populate with none
    if len(genotype_calls) == 0:
        lineage_out_dict['final_genotype'] = ['uncalled']
        lineage_out_dict['name']  = ['NA']
        lineage_out_dict['confidence'] = ['NA']
        lineage_out_dict['lowest support for genotype marker']  = ['']
        lineage_out_dict['poorly supported markers']  = ['']
        lineage_out_dict['max support for additional markers']  = ['']
        lineage_out_dict['additional markers']  = ['']
        lineage_out_dict['node support'] = ['']
    # if there is just one call, list that - but CHECK that all values in heirarchy are good (>0.5)
    # only report up to level in heirarchy where we have a good call
    if len(genotype_calls) == 1:
        # inspect calls for genotype
        best_genotype, confidence, lowest_support_val, poorly_supported_markers, non_matching_support, non_matching_markers, final_markers = inspect_calls(lineage_data['lineage'])
        # extract genotype from lineage thing, add that to the table
        genotype = best_genotype.split('lineage')[1]
        lineage_out_dict['final genotype'] = [genotype]
        lineage_out_dict['name'] = [lineage_name_dict[best_genotype]]

        # fill in all details on confidence/support
        lineage_out_dict['confidence'] = [confidence]
        lineage_out_dict['lowest support for genotype marker'] = [lowest_support_val]
        lineage_out_dict['poorly supported markers'] = ['; '.join(poorly_supported_markers)]
        lineage_out_dict['max support for additional markers'] = [non_matching_support]
        lineage_out_dict['additional markers'] = ['; '.join(non_matching_markers)]
        lineage_out_dict['node support'] = ['; '.join(final_markers)]
        #lineage_out_dict['all_genotype_calls'] = genotype_calls
    # if there is more than one call, we want to report the best, but also other calls
    elif len(genotype_calls) > 1:
        # get the info for each call
        # work out which lineage has the best call
        best_genotype, confidence, lowest_support_val, poorly_supported_markers, non_matching_support, non_matching_markers, final_markers = inspect_calls(lineage_data['lineage'])
        # now write out info
        lineage_out_dict['final genotype'] = [best_genotype.split('lineage')[1]]
        lineage_out_dict['name'] = [lineage_name_dict[best_genotype]]
        lineage_out_dict['confidence'] = [confidence]
        lineage_out_dict['lowest support for genotype marker'] = [lowest_support_val]
        lineage_out_dict['poorly supported markers'] = ['; '.join(poorly_supported_markers)]
        lineage_out_dict['max support for additional markers'] = [non_matching_support]
        lineage_out_dict['additional markers'] = ['; '.join(non_matching_markers)]
        lineage_out_dict['node support'] = ['; '.join(final_markers)]
    # add species info
    lineage_out_dict['species']=['S. sonnei']
    lineage_out_df = pd.DataFrame(lineage_out_dict, columns=['genome', 'species', 'final genotype', 'name', 'confidence', 'lowest support for genotype marker', 'poorly supported markers', 'node support', 'max support for additional markers', 'additional markers'])

    return lineage_out_df, spp_call

def main():
    print("doing something")
    args = get_arguments()
    # list of tables for each result type
    results_tables = []

    # want a single table of outputs
    # create lineage_name_dict, key=mykrobe lineage name, value=human readable name
    lineage_name_dict = {}
    with open(args.alleles, 'r') as lineage_names:
        for line in lineage_names:
            fields = line.strip().split('\t')
            lineage_name_dict[fields[4]] = fields[3]

    # read in json files
    for json_file in args.jsons:
        with open(json_file) as f:
            #with open("ERR017671.mykrobe.json") as f:
            myk_result = json.load(f)
        # get genome name (should first and ONLY key at start of json file)
        if len(list(myk_result.keys())) > 1:
            print("More than one result in mykrobe output file " + json_file + ", quitting")
            sys.exit()
        genome_name = list(myk_result.keys())[0]
        # extract all the data for this genome
        genome_data = myk_result[genome_name]
        # extract the genotyping information
        lineage_data = genome_data["phylogenetics"]
        lineage_table, spp_call = extract_lineage_info(lineage_data, genome_name, lineage_name_dict)
        #all_lineage_tables.append(lineage_table)
        # extract the qrdr information, only if sonnei
        genome_qrdr_table = extract_qrdr_info(genome_data, genome_name, spp_call)
        #all_qrdr_tables.append(genome_qrdr_table)
        # combine the two tables together
        all_info_table = pd.merge(lineage_table, genome_qrdr_table, on="genome", how="inner")
        results_tables.append(all_info_table)


    # concatenate, re-order columns, and write out
    final_results = pd.concat(results_tables, sort=True)
    final_results.to_csv(args.prefix + "_predictResults.tsv", index=False, sep="\t", columns=["genome", "species", "final genotype", "name", "confidence", "num QRDR", "parC_S80I", "gyrA_S83L", "gyrA_S83A", "gyrA_D87G", "gyrA_D87N", "gyrA_D87Y", "lowest support for genotype marker", "poorly supported markers", "max support for additional markers", "additional markers", "node support"])

if __name__ == '__main__':
    main()



import click
import pathlib
import os

from bohra.launcher.Deps import _check_databases


@click.command()
@click.option('--setup_databases',
              is_flag=True,
            #   default=False,
              help="Download and/or setup required databases.")
@click.option('--database_path',
              default="",
              help="If you select to setup databases, specify the path to download them to.")
def init_databases(setup_databases:bool=False, database_path:str=""):
    
    db_install = True if setup_databases else False
    print(f"Will now check your databases{' and install anything that may be required' if db_install else ''}. ")
    _check_databases(db_install=db_install, database_path=database_path)
    


#!/usr/bin/env python3
import sys,  json

output_file = sys.argv[1]

input_file = sys.argv[2:]

list_of_dicts = []
for file in input_file:
    with open(file, 'r') as f:
        data = json.load(f)
        list_of_dicts.append(data)
        # print(data)
print(list_of_dicts)
# with open(output_file, 'w') as f:
#     json.dump(list_of_dicts, f, indent=4)


#!/usr/bin/env python3
import sys, json, datetime, subprocess, pathlib, pandas

files = sys.argv[2:]
df = pandas.DataFrame()
for _file in files:
    print(_file)
    tmp = pandas.read_csv(_file, sep = '\t')
    if df.empty:
        df = tmp
    else:
        df = df.append(tmp)
df = df.fillna('')
df.to_csv(f'{sys.argv[1]}.txt', sep = '\t', index = False)
    



#!/usr/bin/env python3
import pathlib, pandas, math, sys,  re, argparse,json,os,time
def get_date_sequenced(reads):

    ti_c = os.path.getctime(reads)
    ti_c = time.ctime(ti_c)
    t_obj = time.strptime(ti_c)

    seqd = time.strftime("%Y-%m-%d", t_obj)
    return seqd


print(get_date_sequenced(reads = sys.argv[1]))


#!/usr/bin/env python3
import numpy,sys, pathlib, pandas
from sklearn.cluster import AgglomerativeClustering



def extract_unclustered(df, col, col1):

    u = df.groupby(col)[col1].agg('count').reset_index()

    uc = list(u[u[col1] == 1][col])

    return uc


thresholds = [int(i) for i in sys.argv[2].split(',')]
# get distances into lists
tabfile = sys.argv[1]
if pathlib.Path(tabfile).exists():
    isos = []
    mat = []
    if isinstance(tabfile, str):
        print(f"opening {tabfile}")
        with open(tabfile, 'r') as d:
            tab = d.read().split('\n')

            # get isolates
            for line in tab[1:len(tab)]:
                ln = line.split('\t')
                if len(ln) > 1:
                    # isos.append(ln[0])
                    mat.append(ln[1:len(ln)])

            isos = tab[0].split('\t')[1:]
    print(len(mat))
    clusters = pandas.DataFrame()
    # convery the lists to a numpy array
    print("converting matrix to numpy array")
    X = numpy.array(mat)
    X = X.astype(numpy.float64)

    for level in thresholds:
        print(f"Clustering at {level}")
        clustering = AgglomerativeClustering(n_clusters = None, metric = 'precomputed',linkage = f"{sys.argv[3]}", distance_threshold =int(level)).fit(X)
        df = pandas.DataFrame(data = {'ID': isos, f"Tx:{level}": clustering.labels_})
        print(len(clustering.labels_))
        print(len(isos))
        print(df.shape) 
        colname = f"Tx:{level-1}"
        df = df.rename(columns={f"Tx:{level}": colname})
        print(len(df[f"{colname}"].unique()))
        df[f"{colname}"] = df[f"{colname}"] + 1
        df = df.fillna('')
        df[f"{colname}"] = df[f"{colname}"].apply(lambda x: f"{x}")
        print(df.shape)

        uc = extract_unclustered(df = df, col = f"{colname}", col1 = 'ID')
        df[f"{colname}"] = numpy.where(df[f"{colname}"].isin(uc), 'UC', df[f"{colname}"])
        df = df.drop_duplicates(subset=['ID'])
        if clusters.empty:
            clusters = df
        else:
            clusters = df.merge(clusters, right_on = 'ID',left_on ='ID', how = 'inner')
        clusters = clusters.drop_duplicates()
        print(clusters.shape)
    clusters = clusters.fillna('')

    clusters.to_csv(f'clusters.txt', sep = '\t', index = False)
